@@ -28,6 +28,7 @@ def dot_to_underscore(instring):
 
 class RouterCompiler(object):
     lo_interface = "lo0" #make this clear distinction between interface id and lo IP
+    lo_interface_prefix = "lo"
 # and set per platform
 
     """Base Router compiler"""
@@ -47,6 +48,7 @@ class RouterCompiler(object):
         node.loopback = ipv4_node.loopback
         node.loopback_subnet = netaddr.IPNetwork(node.loopback)
         node.loopback_subnet.prefixlen = 32
+
         self.interfaces(node)
         if node in self.anm['ospf']:
             self.ospf(node)
@@ -72,7 +74,6 @@ class RouterCompiler(object):
                 ipv6_link = G_ipv6.edge(link)
                 ipv6_subnet =  ipv6_link.dst.subnet # netmask comes from collision domain on the link
                 ipv6_address = address_prefixlen_to_network(ipv6_link.ip, ipv6_subnet.prefixlen)
-
             
             node.interfaces.append(
                     _edge_id = link.edge_id, # used if need to append
@@ -85,6 +86,21 @@ class RouterCompiler(object):
                     physical = True,
                     )
 
+        for index, interface in enumerate(phy_node.interfaces("is_loopback")):
+            ip_interface = G_ipv4.node(node).interface(interface)
+            vrf_interface = self.anm['vrf'].node(node).interface(interface)
+            index = index + 1 # loopback0 (ie index 0) is reserved
+            id = "%s%s" % (self.lo_interface_prefix, index)
+            node.interfaces.append(
+                    id = id,
+                    description = interface.description,
+                    ipv4_address = ip_interface.loopback,
+                    ipv4_subnet = node.loopback_subnet,
+                    vrf_name = vrf_interface.vrf_name,
+                    route_target = interface.route_target,
+                    )
+
+        #TODO: work out why this doesn't sort loopbacks properly (because append loopback 0 last)
         node.interfaces.sort("id")
     
     def ospf(self, node):
@@ -164,6 +180,8 @@ class RouterCompiler(object):
                 #TODO: fix this: this is a workaround for Quagga next-hop denied for loopback (even with static route)
                 ip_link = G_ipv4.edge(session)
                 dst_int_ip = G_ipv4.edges(ip_link.dst, neigh).next().ip_address #TODO: split this to a helper function
+
+                #TODO: make this a returned value
                 node.bgp.ebgp_neighbors.append( {
                     'neighbor': neigh.label,
                     'use_ipv4': use_ipv4,
@@ -205,7 +223,7 @@ class QuaggaCompiler(RouterCompiler):
         if phy_node.is_router:
             node.interfaces.append(
                     id = self.lo_interface,
-                    description = "Loopback for BGP",
+                    description = "Loopback",
                     ipv4_address = ipv4_node.loopback,
                     ipv4_subnet = node.loopback_subnet
                     )
@@ -239,17 +257,17 @@ class QuaggaCompiler(RouterCompiler):
 class IosBaseCompiler(RouterCompiler):
     """Base IOS compiler"""
 
-    lo_interface = "Loopback0"
+    lo_interface_prefix = "Loopback"
+    lo_interface = "%s%s"  % (lo_interface_prefix, 0)
 
     def compile(self, node):
         super(IosBaseCompiler, self).compile(node)
         if node in self.anm['isis']:
             self.isis(node)
         node.label = self.anm['phy'].node(node).label
-
+        self.vrf(node)
         
     def interfaces(self, node):
-
         ipv4_cidr = ipv6_address = ipv4_address = ipv4_loopback_subnet = None
         if node.ip.use_ipv4:
             ipv4_node = self.anm['ipv4'].node(node)
@@ -296,6 +314,15 @@ class IosBaseCompiler(RouterCompiler):
 
     def ospf(self, node):
         super(IosBaseCompiler, self).ospf(node)
+        
+    def vrf(self, node):
+        vrf_node = self.anm['vrf'].node(node)
+        for loopback in vrf_node.interfaces("is_loopback", "vrf_name"):
+            continue
+            if loopback: # ie if any data set
+                print "vrf lo", loopback
+            else:
+                print "no vrf lo", loopback
 
     def isis(self, node):
         #TODO: this needs to go into IOS2 for neatness
